/**
 * Classe Renderer2D, junto com a classe JanelaExemploJava2D, mostra um exemplo de 
 * como trabalhar com Java e OpenGL utilizando a API JOGL.  aberta uma 
 * janela na qual  desenhada uma casinha, possibilitando uma interao do usurio.
 * 
 * @author Isabel H. Manssour e Marcelo Cohen  
 * @version 1.0
 */

import java.awt.event.*; 

import javax.media.opengl.*;
import javax.media.opengl.glu.*;

public class Renderer2D extends KeyAdapter implements GLEventListener
{
	// Atributos
	private GL gl;
	private GLU glu;
	private GLAutoDrawable glDrawable;
	private float translacaoX, translacaoY;
	private double left, right, bottom, top;
	private float panX, panY;
	private int largura, altura;

	/**
	 * Mtodo definido na interface GLEventListener e chamado pelo objeto no qual ser feito o desenho
	 * logo aps a inicializao do contexto OpenGL. 
	 */    
	public void init(GLAutoDrawable drawable)
	{
		left = -1.0;
		right = 1.0;
		bottom = -1.0;
		top = 1.0;
		panX = panY = 0;
		glDrawable = drawable;
		gl = drawable.getGL();     
		glu = new GLU();

		drawable.setGL(new DebugGL(gl));        

		gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
		
		// Define a janela de visualizao 2D
		gl.glMatrixMode(GL.GL_PROJECTION);
		glu.gluOrtho2D(left+panX, right+panX, bottom+panY, top+panY);
		gl.glMatrixMode(GL.GL_MODELVIEW);
	}
	
	/**
	 * Mtodo que desenha os eixos do plano cartesiano.
	 */
	public void desenhaEixos()
	{
		gl.glLineWidth(1); // Determina a espessura da linha que ser desenhada
		gl.glBegin(GL.GL_LINES);
			gl.glVertex2d(left, 0);
			gl.glVertex2d(right,0);
			gl.glVertex2d( 0, bottom);
			gl.glVertex2d( 0, top);
		gl.glEnd(); 
	}

	/**
	 * Mtodo que desenha uma casinha no centro da janela.
	 */	
	public void desenhaCasinha()
	{
		gl.glLineWidth(3); // Determina a espessura da linha que ser desenhada               
		gl.glBegin(GL.GL_TRIANGLES);
			gl.glColor3f(0.0f, 0.0f, 1.0f);                   
			gl.glVertex2f(-0.2f,0.1f);
			gl.glColor3f(1.0f, 0.0f, 0.0f);                     
			gl.glVertex2f(0.0f,0.22f);
			gl.glColor3f(0.0f, 0.0f, 1.0f);                     
			gl.glVertex2f(0.2f,0.1f);
		gl.glEnd();     

		gl.glBegin(GL.GL_LINE_LOOP);
			gl.glVertex2f(-0.2f,-0.2f);                   
			gl.glVertex2f(-0.2f,0.1f);
			gl.glVertex2f(0.2f,0.1f);
			gl.glVertex2f(0.2f,-0.2f);            
		gl.glEnd(); 		
	}

	/**
	 * Mtodo definido na interface GLEventListener e chamado pelo objeto no qual ser feito o desenho
	 * para comear a fazer o desenho OpenGL pelo cliente.
	 */  
	public void display(GLAutoDrawable drawable)
	{
		gl.glMatrixMode(GL.GL_PROJECTION);
		gl.glLoadIdentity();
		// Estabelece a janela de seleo (esquerda, direita, inferior, 
		// superior) mantendo a proporo com a janela de visualizao
		if (largura <= altura) 
			glu.gluOrtho2D (left+panX, right+panX, bottom*altura/largura+panY, top*altura/largura+panY);
		else 
			glu.gluOrtho2D (left*largura/altura+panX, right*largura/altura+panX, bottom+panY, top+panY);
		gl.glMatrixMode(GL.GL_MODELVIEW);
		
		gl.glClear(GL.GL_COLOR_BUFFER_BIT);
		gl.glLoadIdentity();    

		gl.glColor3ub((byte)0, (byte)0, (byte)255);

		desenhaEixos();
		
		// Aplica transformaes geomtricas sobre a casinha
		gl.glTranslatef(translacaoX, translacaoY, 0f);
		desenhaCasinha();              
	}

	/**
	 * Mtodo definido na interface GLEventListener e chamado pelo objeto no qual ser feito o desenho
	 * depois que a janela foi redimensionada.
	 */  
	public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height)
	{
		largura = width;
		altura = height;
		
		gl.glViewport(0, 0, width, height);      

		// Inicializa o sistema de coordenadas
		gl.glMatrixMode(GL.GL_PROJECTION);
		gl.glLoadIdentity();

		// Estabelece a janela de seleo (esquerda, direita, inferior, 
		// superior) mantendo a proporo com a janela de visualizao
		if (largura <= altura) 
			glu.gluOrtho2D (left+panX, right+panX, bottom*altura/largura+panY, top*altura/largura+panY);
		else 
			glu.gluOrtho2D (left*largura/altura+panX, right*largura/altura+panX, bottom+panY, top+panY);

		gl.glMatrixMode(GL.GL_MODELVIEW);		
	}

	/**
	 * Mtodo definido na interface GLEventListener e chamado pelo objeto no qual ser feito o desenho
	 * quando o modo de exibio ou o dispositivo de exibio associado foi alterado.
	 */  
	public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) { }

	/**
	 * Mtodo definido na interface KeyListener que est sendo implementado que seja 
	 * feita a sada do sistema quando for pressionada a tecla ESC.
	 */        
	public void keyPressed(KeyEvent e)
	{
		switch (e.getKeyCode())
		{	
			case KeyEvent.VK_RIGHT:	translacaoX+=0.1;   
									break;
			case KeyEvent.VK_LEFT:	translacaoX-=0.1;   
									break;     
			case KeyEvent.VK_UP:	translacaoY+=0.1;   
									break;                              
			case KeyEvent.VK_DOWN:	translacaoY-=0.1;   
									break; 		
			case KeyEvent.VK_HOME:	left -= 0.1;  
									right += 0.1;
									bottom -= 0.1;
									top += 0.1;
									break; 	
			case KeyEvent.VK_END:	left += 0.1;  
									right -= 0.1;
									bottom += 0.1;
									top -= 0.1;
									break; 				
			case KeyEvent.VK_L:		panX+=0.1;   
									break;
			case KeyEvent.VK_R:		panX-=0.1;   
									break;     
			case KeyEvent.VK_T:		panY+=0.1;   
									break;                              
			case KeyEvent.VK_B:		panY-=0.1;   
									break; 
			case KeyEvent.VK_ESCAPE:System.exit(0);
									break;
		}  
		glDrawable.display();
	}

}

